//
//  CommunityCategoryListView.swift
//  SampleApp
//
//  Created by Nishan Niraula on 8/3/20.
//  Copyright © 2020 David Zhang. All rights reserved.
//

import SwiftUI
import AmitySDK

// Using LiveCollection as Observable Object in SwiftUI view.
struct CommunityCategoryListView: View {
    
    @ObservedObject var manager: CommunityCategoryManager = CommunityCategoryManager()
    
    enum FilterType {
        case sort
        case isDeleted
    }
    
    enum ActionSheetType {
        case filter(FilterType)
        case other
    }
    
    @State var categoryId: String = ""
    @State var showActionSheet = false
    @State var actionSheetType: ActionSheetType = .filter(.sort)
    @State var sortedBy: String = "Display Name"
    @State var includeDeletedValue = "NO"
    @State var shouldIncludeDeleted: Bool = false
    
    var body: some View {
        NavigationView {
            VStack(alignment: .leading) {
                HStack {
                    SelectionButton(title: "Sorted By", selection: $sortedBy) {
                        self.actionSheetType = .filter(.sort)
                        self.showActionSheet.toggle()
                    }
                    
                    Spacer()
                    
                    SelectionButton(title: "Include Deleted Categories", selection: $includeDeletedValue) {
                        self.actionSheetType = .filter(.isDeleted)
                        self.showActionSheet.toggle()
                    }
                    .onChange(of: shouldIncludeDeleted) { newValue in
                        includeDeletedValue = newValue ? "YES" : "NO"
                    }
                }
                .padding()
                
                CategoryCollectionView(categoryCollection: manager.categoryCollection)
            }
            .navigationBarTitle("Community Categories", displayMode: .inline)
        }
        .actionSheet(isPresented: $showActionSheet, content: { () -> ActionSheet in
            switch self.actionSheetType {
            case .filter(let filterType) :
                if filterType == .isDeleted {
                    return ActionSheet(title: Text(""), message: Text("Sort Categories"), buttons: [
                        
                        .default(Text("Include Deleted Categories"), action: {
                            self.shouldIncludeDeleted = true
                            self.manager.filterCategories(includeDeleted: true)
                        }),
                        .default(Text("Don't include deleted categories"), action: {
                            self.shouldIncludeDeleted = false
                            self.manager.filterCategories(includeDeleted: true)
                        }),
                        .cancel()
                    ])
                } else {
                    return ActionSheet(title: Text(""), message: Text("Sort Categories"), buttons: [
                        
                        .default(Text("Display Name"), action: {
                            self.manager.sortCategories(sortOption: .displayName)
                            self.sortedBy = "Display Name"
                        }),
                        .default(Text("First Created"), action: {
                            self.manager.sortCategories(sortOption: .firstCreated)
                            self.sortedBy = "First Created"
                        }),
                        .default(Text("Last Created"), action: {
                            self.manager.sortCategories(sortOption: .lastCreated)
                            self.sortedBy = "Last Created"
                        }),
                        .cancel()
                    ])
                }
            case .other:
                return ActionSheet(title: Text(""), message: Text("Actions"), buttons: [
                    .default(Text("Copy Category Id"), action: {
                        UIPasteboard.general.string = self.categoryId
                    }),
                    .cancel()
                ])
            }
        })
    }
}

struct CategoryCollectionView: View {
    
    // Live collection as observable object
    @ObservedObject var categoryCollection: AmityCollection<AmityCommunityCategory>
    
    @State private var showActionSheet = false
    @State private var copiedCategoryId = ""
    
    var body: some View {
        List(categoryCollection.snapshots, id: \.categoryId) { item in
            VStack(alignment: .leading) {
                Text("Name: \(item.name)")
                    .padding(.top, 4)
                Text("Id: \(item.categoryId)")
                    .font(.footnote)
                    .foregroundColor(.secondary)
                    .padding(.top, 4)
                Text("Avatar: \(item.avatarFileId)")
                    .asDescription()
            }
            .onTapGesture {
                self.copiedCategoryId = item.categoryId
                self.showActionSheet.toggle()
            }
            .onAppear {
                if let lastCategory = categoryCollection.snapshots.last, lastCategory.categoryId == item.categoryId {
                    if categoryCollection.hasNext {
                        categoryCollection.nextPage()
                    }
                }
            }
        }
        .actionSheet(isPresented: $showActionSheet) {
            ActionSheet(title: Text(""), message: Text("Actions"), buttons: [
                .default(Text("Copy Category Id"), action: {
                    UIPasteboard.general.string = self.copiedCategoryId
                }),
                .cancel()
            ])
        }
    }
}

struct CommunityCategoryListView_Previews: PreviewProvider {
    
    static var previews: some View {
        CommunityCategoryListView()
    }
}


struct DescriptionStyle: ViewModifier {
    
    func body(content: Content) -> some View {
        return content
            .font(.footnote)
            .foregroundColor(.secondary)
            .padding(.top, 2)
    }
}

extension View {
    
    func asDescription() -> some View {
        return self.modifier(DescriptionStyle())
    }
}

class CommunityCategoryManager: ObservableObject {
    
    private let communityRepository = AmityCommunityRepository(client: AmityManager.shared.client!)
    private var sortOption: AmityCommunityCategoriesSortOption = .displayName
    private var includeDeletedCategories = false
    
    var categoryCollection: AmityCollection<AmityCommunityCategory>
    
    init() {
        categoryCollection = communityRepository.getCategories(sortBy: .displayName, includeDeleted: false)
    }
    
    func sortCategories(sortOption: AmityCommunityCategoriesSortOption) {
        self.sortOption = sortOption
        
        switch sortOption {
        case .displayName:
            categoryCollection = communityRepository.getCategories(sortBy: .displayName, includeDeleted: includeDeletedCategories)
        case .firstCreated:
            categoryCollection = communityRepository.getCategories(sortBy: .firstCreated, includeDeleted: includeDeletedCategories)
        case .lastCreated:
            categoryCollection = communityRepository.getCategories(sortBy: .lastCreated, includeDeleted: includeDeletedCategories)
        @unknown default:
            fatalError()
        }
    }
    
    func filterCategories(includeDeleted: Bool) {
        self.includeDeletedCategories = includeDeleted
        
        categoryCollection = communityRepository.getCategories(sortBy: sortOption, includeDeleted: includeDeletedCategories)
    }
}
