//
//  CommunityMemberView.swift
//  SampleApp
//
//  Created by Michael Abadi on 07/08/20.
//  Copyright © 2020 David Zhang. All rights reserved.
//

import SwiftUI
import AmitySDK

struct CommunityMemberView: View {
    
    @StateObject var viewModel: CommunityMemberViewModel
    @State var roleTitle: String = ""
    @State var userIdTitle: String = ""
    @State var keywordTitle: String = ""
    @State var membershipTitle: String = ""
    @State var sortByTitle: String = ""
    
    init(communityId: String) {
        _viewModel = StateObject(wrappedValue: CommunityMemberViewModel(communityId: communityId))
    }
    
    var body: some View {
        VStack {
            
            HStack {
                TextField("User id", text: $userIdTitle)
                    .autocapitalization(.none)
                    .textFieldStyle(RoundedBorderTextFieldStyle())
                
                Button("Ban") {
                    let enteredUserId = userIdTitle.trimmingCharacters(in: .whitespacesAndNewlines)
                    let userId = enteredUserId.isEmpty ? [] : enteredUserId.components(separatedBy: ",")
                    viewModel.banUser(userId: userId)
                }
                .foregroundColor(.red)
                
                Button("Unban") {
                    let enteredUserId = userIdTitle.trimmingCharacters(in: .whitespacesAndNewlines)
                    let userId = enteredUserId.isEmpty ? [] : enteredUserId.components(separatedBy: ",")
                    viewModel.unBan(userId: userId)
                }
            }
            .padding()
            
            Text("Result: \(viewModel.banUnbanUpdateStatus)")
            
            Divider()
            
            HStack {
                TextField("Enter Keyword", text: $keywordTitle)
                    .autocapitalization(.none)
                    .textFieldStyle(RoundedBorderTextFieldStyle())
                
                TextField("Enter Role", text: $roleTitle)
                    .autocapitalization(.none)
                    .textFieldStyle(RoundedBorderTextFieldStyle())
            }
            .padding(.horizontal)
            
            Text("You can also enter multiple roles separated by comma i.e ,")
                .font(.caption)
            Divider()
            
            HStack {
                PickerSelection(title: "Membership", selection: $membershipTitle, options: viewModel.membership)
                
                PickerSelection(title: "SortBy", selection: $sortByTitle, options: viewModel.sortBy)
                    .disabled(!keywordTitle.isEmpty)
            }
            
            Button("Query Members") {
                let enteredRoles = roleTitle.trimmingCharacters(in: .whitespacesAndNewlines)
                let roles = enteredRoles.isEmpty ? [] : enteredRoles.components(separatedBy: ",")
                
                
                let sortBy: AmityCommunityMembershipSortOption
                switch sortByTitle {
                case "lastCreated":
                    sortBy = .lastCreated
                case "firstCreated":
                    sortBy = .firstCreated
                default:
                    sortBy = .lastCreated
                }
                if keywordTitle.isEmpty {
                    let membership: AmityCommunityMembership.QueryFilter
                    switch membershipTitle {
                    case "all":
                        membership = .all
                    case "member":
                        membership = .member
                    case "banned":
                        membership = .banned
                    default:
                        membership = .member
                    }
                    viewModel.getMembers(membership: membership, roles: roles, sortBy: sortBy)
                } else {
                    let membership: [AmityCommunityMembership.SearchFilter]
                    switch membershipTitle {
                    case "all":
                        membership = [.banned, .member]
                    case "member":
                        membership = [.member]
                    case "banned":
                        membership = [.banned]
                    default:
                        membership = [.member]
                    }
                    viewModel.searchMembers(keyword: keywordTitle, membership: membership, roles: roles)
                }
            }
            
            CommunityMemberCollectionView(memberCollection: viewModel.memberCollection, viewModel: viewModel)
        }
        .onAppear(perform: {
            viewModel.getMembers(membership: .member, roles: [], sortBy: .lastCreated)
        })
    }
}

struct CommunityMemberView_Previews: PreviewProvider {
    static var previews: some View {
        CommunityMemberView(communityId: "")
    }
}

struct CommunityRoleView: View {
    @State var roles: [String] = []
    @State var enteredRole: String = ""
    @State var selectedPermission: Int = 0
    @State var rolesText = ""
    
    let viewModel: CommunityMemberViewModel
    let userId: String
    
    init(viewModel: CommunityMemberViewModel, userId: String) {
        self.viewModel = viewModel
        self.userId = userId
    }
    
    var body: some View {
        Form {
            Section(header: Text("Roles")) {
                TextField("Enter role", text: $enteredRole)
                    .autocapitalization(.none)
                
                Text(rolesText.isEmpty ? "The roles to assign/unassign" : rolesText)
                
                Button("Add Role") {
                    if self.roles.contains(self.enteredRole) || self.enteredRole.isEmpty {
                        return
                    }
                    self.roles.append(self.enteredRole)
                    self.rolesText.append("\(self.roles.count > 1 ? ", " : "")\(self.enteredRole)")
                    self.enteredRole = ""
                }
                
                Button("Assign Role") {
                    self.viewModel.addRoles(roles: self.roles, userId: userId)
                    self.roles = []
                    self.rolesText = ""
                }
                
                Button("UnAssign Role") {
                    self.viewModel.removeRoles(roles: self.roles, userId: userId)
                    self.roles = []
                    self.rolesText = ""
                }
                
                Text("Result: \(viewModel.roleUpdateStatus)")
                    .foregroundColor(Color.red)
                    .multilineTextAlignment(.center)
            }
            
            Section(header: Text("Permissions"), footer: Text("This checks permissions for current logged in user")) {
                Picker(selection: $selectedPermission, label: Text("Select Permission")) {
                    ForEach(0..<SDKPermission.permissions.count, id: \.self) { i in
                        Text(SDKPermission.permissions[i].identifier)
                    }
                }
                
                Button("Check Permission") {
                    self.viewModel.checkPermission(permission: SDKPermission.permissions[selectedPermission])
                }
                
                Text("Result: \(viewModel.permissionStatus)")
                    .foregroundColor(Color.red)
                    .multilineTextAlignment(.center)
            }
        }.onAppear(perform: {
            viewModel.resetStatus()
        })
    }
}

struct CommunityMemberCollectionView: View {
    
    @ObservedObject var memberCollection: AmityCollection<AmityCommunityMember>
    let viewModel: CommunityMemberViewModel
    
    var body: some View {
        List(memberCollection.snapshots, id: \.userId) { item in
            
            let m = CommunityMember(member: item)
            
            NavigationLink(
                destination: CommunityRoleView(viewModel: viewModel, userId: m.memberUserId),
                label: {
                    VStack(alignment: .leading) {
                        Text("\(m.displayName)")
                        Text("Role: \(m.roles)")
                            .font(.body)
                            .padding(.top, 2)
                            .foregroundColor(Color.secondary)
                    }
                })
        }
    }
}
