//
//  StoryCollectionView.swift
//  SampleApp
//
//  Created by Manuchet Rungraksa on 4/12/2566 BE.
//  Copyright © 2566 BE David Zhang. All rights reserved.
//

import Foundation
import SwiftUI
import AmitySDK

struct StoryCollectionView: View  {
        
    @ObservedObject var storyCollection: AmityCollection<AmityStory>
    @ObservedObject var storyTargetObject: AmityObject<AmityStoryTarget>
    let viewModel: StoryListViewModel
    
    @Environment(\.navigationController) var navigationController
    @State private var selection: String?
    @State private var showAlert = false
    @State private var addReactionAlert = false
    @State private var alertText = ""
    @State var reachUserListView: ReachUserListView?
    @State var reactionTestView: ReactionTestView?
    @State var storyRTEView: StoryRTEView?
    

    var body: some View {
        if viewModel.targetId != nil {
            Text("Has unseen story: \(storyTargetObject.snapshot?.hasUnseen ?? false ? "Yes" :  "No")")
            Text("Syncing stoies count: \(storyTargetObject.snapshot?.syncingStoriesCount ?? 0)")
            Text("Failed stories count: \(storyTargetObject.snapshot?.failedStoriesCount ?? 0)")
        }
        List(storyCollection.snapshots, id: \.storyId) { item in
            let story = StoryModel(story: item)
            storyView(story)
        }
        .listStyle(.plain)
    }
    
    func storyView(_ story: StoryModel) -> some View {
        ZStack {
            
            VStack {
                
                VStack(spacing: 5) {
                    
                    let data = story.getPreviewData()
                    
                    ForEach(0..<data.count, id: \.self) { index in
                        let item = data[index]
                        CardTextView(key: item.0, value: item.1, lineLimit: nil)
                    }
                    
                }
                .padding([.top, .bottom], 16)
            }
            .background(Color(.pastelBlue))
            .cornerRadius(20)
            .shadow(color: Color.gray.opacity(0.6), radius: 8, x: 0, y: 0)
            .padding([.top, .bottom], 8)
            .contextMenu {
                Menu("Delete") {
                    Button("Hard Delete") {
                        deleteStory(storyId: story.id, hardDelete: true)
                    }
                    Button("Soft Delete") {
                        deleteStory(storyId: story.id, hardDelete: false)
                    }
                }
                Menu("Analytics") {
                    Button("Mark seen") {
                        viewModel.markSeen(story: story.storyObject)
                    }
                    Button("Get Reach Users") {
                        let viewmodel = ReachUserListViewModel(viewedId: story.storyId, viewedType: .story, impressionCount: story.impression, reachCount: story.reach)
                        reachUserListView = ReachUserListView(viewModel: viewmodel)

                        self.selection = story.storyId + "+story"
                    }
                    Button("Mark link as clicked") {
                        viewModel.markLinkAsClicked(story: story.storyObject)
                    }
                }
//                Button("Comment View") {
//                    let feedStoryboard = UIStoryboard(name: "Feed", bundle: nil)
//                    let controller = feedStoryboard.instantiateViewController(withIdentifier: CommentsViewController.identifier) as! CommentsViewController
//
//                    controller.story = story
//                    guard let client = AmityManager.shared.client else { return }
//                    controller.commentManager = UserPostCommentManager(client: client, postId: story.storyId, parentCommentId: nil, userId: client.currentUserId, userName: client.user?.snapshot?.displayName, communityId: nil, referenceType: .story)
//
//                    self.navigationController?.pushViewController(controller, animated: true)
//                }
//                Menu("Reactions") {
//                    Button("Add Reaction") {
//                        addReactionAction(story: story.storyObject)
//                    }
//                    Button("Remove Reaction") {
//                        removeReactionAction(story: story.storyObject)
//                    }
//                    Button("Show My Reactions") {
//                        showMyReactionAction(story: story.storyObject)
//                    }
//                    Button("Show All Reactions") {
//                        showAllReactions(story: story.storyObject)
//                    }
//                    Button("Test Reaction") {
//                        let viewModel = ReactionTestViewModel(messageId: story.storyId, type: .story)
//                        reactionTestView = ReactionTestView(viewModel: viewModel)
//                        self.selection = story.storyId + "+reactionTest"
//
//                    }
//                }
//                Button("RTE") {
//                    let viewModel = StoryRTEViewModel(story: story.storyObject)
//                    storyRTEView = StoryRTEView(viewModel: viewModel)
//                    self.selection = story.storyId + "+RTE"
//                }
                Button("Copy story id") {
                    UIPasteboard.general.string = story.storyId
                }
            }
            .alert(isPresented: $showAlert) {
                Alert(
                    title: Text(alertText)
                )
            }
            
            NavigationLink(
                destination: reachUserListView,
                tag: story.storyId + "+story",
                selection: self.$selection,
                label: {
                    EmptyView()
                })
            .buttonStyle(PlainButtonStyle())
            .hidden()
            
            NavigationLink(
                destination: reactionTestView,
                tag: story.storyId + "+reactionTest",
                selection: self.$selection,
                label: {
                    EmptyView()
                })
            .buttonStyle(PlainButtonStyle())
            .hidden()
            
            NavigationLink(
                destination: storyRTEView,
                tag: story.storyId + "+RTE",
                selection: self.$selection,
                label: {
                    EmptyView()
                })
            .buttonStyle(PlainButtonStyle())
            .hidden()
        }
    }
    
    func addReactionAction(story: AmityStory) {

        guard let vc = AmityAddReactionsViewController.makeViewController() as? AmityAddReactionsViewController else { return }
        vc.setupForStory(type: .add, story: story) { text, story in
            addReaction(storyId: story.storyId, reaction: text)
        }
        let nvc = UINavigationController(rootViewController: vc)
        navigationController?.present(nvc, animated: true, completion: nil)
        
        
    }

    func removeReactionAction(story: AmityStory) {
        guard let vc = AmityAddReactionsViewController.makeViewController() as? AmityAddReactionsViewController else { return }
        vc.setupForStory(type: .remove, story: story) { text, story in
            removeReaction(storyId: story.storyId, reaction: text)
        }
        let nvc = UINavigationController(rootViewController: vc)
        navigationController?.present(nvc, animated: true, completion: nil)
    }

    func showMyReactionAction(story: AmityStory) {
//        let vc = AmityReactionsViewController.makeViewController()
//        let viewModels: [ReactionViewModel] = story.myReactions.map { reaction in
//            ReactionViewModel(reactionName: reaction, userDisplayName: "me", createdAt: nil)
//        }
//        let dataSource = StaticReactionDatasource(viewModels: viewModels)
//        vc.setDatasource(dataSource)
//        let nvc = UINavigationController(rootViewController: vc)
//        navigationController?.present(nvc, animated: true, completion: nil)
    }

    func showAllReactions(story: AmityStory) {
//        let vc = AmityReactionsViewController.makeViewController()
//        
//        let reactionsCollection = viewModel.storyManager.reactionRepository.getReactions(
//            story.storyId,
//            referenceType: .story,
//            reactionName: nil
//        )
//        
//        let dataSource = LiveReactionDatasource(reactionsCollection: reactionsCollection)
//        vc.setDatasource(dataSource)
//        let nvc = UINavigationController(rootViewController: vc)
//        navigationController?.present(nvc, animated: true, completion: nil)
    }

    func deleteStory(storyId: String, hardDelete: Bool) {
        Task { @MainActor in
            do {
                try await viewModel.deleteStory(storyId: storyId, hardDelete: hardDelete)
                
                alertText = "Story successfully Deleted"
                showAlert = true
                
            } catch {
                alertText = error.localizedDescription
                showAlert = true
            }
        }
    }
    
    func addReaction(storyId: String, reaction: String) {
        Task { @MainActor in
            do {
                let _ = try await viewModel.addReaction(storyId: storyId, reaction: reaction)
                
                alertText = "Successfully add \(reaction) reaction"
                showAlert = true
            } catch {
                alertText = error.localizedDescription
                showAlert = true
            }
        }
    }
    
    func removeReaction(storyId: String, reaction: String) {
        Task { @MainActor in
            do {
                let _ = try await viewModel.removeReaction(storyId: storyId, reaction: reaction)
                
                alertText = "Successfully remove \(reaction) reaction"
                showAlert = true
            } catch {
                alertText = error.localizedDescription
                showAlert = true
            }
        }
    }
}

// Concrete implementor
class StoryListViewModel: ObservableObject {
    
    var pageTitle: String
    var targetId: String?
    
    private let storyRepository = AmityStoryRepository(client: AmityManager.shared.client!)
    
    @Published var storyCollection: AmityCollection<AmityStory>
    @Published var storyTargetObject: AmityObject<AmityStoryTarget>
        
    var storyTargets: [StoryTargetListModel] = []

    var storyManager = StoryManager()
    
    var currentSortOption: AmityStoryQuerySortOption = .lastCreated
    
    @MainActor
    init(targetId: String) {
        self.pageTitle = "Story in \(targetId)"
        self.targetId = targetId
        storyCollection = storyManager.getActiveStories(communityId: targetId, sortOptions: currentSortOption)
        storyTargetObject = storyManager.getStoryTarget(communityId: targetId)
    }
    
    @MainActor
    init() {
        self.pageTitle = "Stories by targets"
//        storyCollection = storyManager.getStoriesByTargets(searchTargets: [], sortOptions: currentSortOption)
        storyCollection = storyManager.getActiveStories(communityId: "", sortOptions: currentSortOption)
        storyTargetObject = storyManager.getStoryTarget(communityId: "")
    }
    
    @MainActor
    func queryStory() {
        guard let targetId else { return }
        storyCollection = storyManager.getActiveStories(communityId: targetId, sortOptions: currentSortOption)
    }
    
    @MainActor
    func queryStoryByTargets(textTarget: String) {
        var searchTargets = [StoryTargetSearchInfo]()
        let eachTarget = textTarget.components(separatedBy: ", ")
        for target in eachTarget {
            let splited = target.components(separatedBy: "/")
            
            let searchTarget = StoryTargetSearchInfo(targetType: AmityStoryTargetType(rawValue: splited.first ?? "") ?? .community, targetId: splited.last ?? "")
            searchTargets.append(searchTarget)
        }
        
//        storyCollection = storyManager.getStoriesByTargets(searchTargets: searchTargets, sortOptions: currentSortOption)
        storyCollection = storyManager.getActiveStories(communityId: "", sortOptions: currentSortOption)
    }
    
    @MainActor 
    func queryStoryTarget() {
        guard let targetId else { return }
        storyTargetObject = storyManager.getStoryTarget(communityId: targetId)
        
    }
    
    func setSort(_ sort: AmityStoryQuerySortOption) {
        self.currentSortOption = sort
    }
        
    func deleteStory(storyId: String, hardDelete: Bool) async throws {
        try await storyManager.deleteStory(storyId: storyId, hardDelete: hardDelete)
    }
    
    func markSeen(story: AmityStory) {
        story.analytics.markAsSeen()
    }
    
    func markLinkAsClicked(story: AmityStory) {
        story.analytics.markLinkAsClicked()
    }
    
    func addReaction(storyId: String, reaction: String) async throws -> Bool {
        return try await storyManager.addReaction(storyId: storyId, reaction: reaction)
    }
    
    func removeReaction(storyId: String, reaction: String) async throws -> Bool {
        return try await storyManager.removeReaction(storyId: storyId, reaction: reaction)
    }
    
}
